import { DatePipe } from '@angular/common';
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Owner } from '../../../../models/owner.model';
import { Structure } from '../../../../models/structure.model';
import { Module } from '../../../../structure-list/models/module.model';
import { Utils } from '../../../../utils/utils';
import { RecapsType } from '../../enums/orientation.enums';

@Component({
  selector: 'app-orientation-recap',
  templateUrl: './orientation-recap.component.html',
  styleUrls: ['./orientation-recap.component.scss'],
})
export class OrientationRecapComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  @Input() recapType: RecapsType;
  @Input() socialWorker?: Owner;
  @Input() structureRDV?: Structure;
  @Input() isToPrint = true;
  @Output() checkValidation = new EventEmitter<any>();

  public utils = new Utils();
  public needs: Module[];
  public structuresToPrint: Structure[];
  public orientator: { structureName: string; structureMail: string; structurePhone: string };
  public comment: string;
  public beneficiary: { name: string; surname: string };
  public recap: { day: string; hours: string; month: string; language?: string };

  constructor(private datePipe: DatePipe) {}

  ngOnInit(): void {
    this.checkValidation.emit();
    // When requesting for an appointments, we save Filters within the form 'onlineDemarcheType',
    // instead of Modules within the form 'filters'. This harmonizes it.
    if (this.recapType === RecapsType.appointment) {
      this.needs = this.utils.convertFiltersToModule(this.form.get('onlineDemarcheType').value);
    } else {
      this.needs = this.form.get('filters').value;
    }
    this.beneficiary = {
      name: this.form.get('name').value,
      surname: this.form.get('surname').value,
    };
    switch (this.recapType) {
      case RecapsType.onlineMediation:
        this.handleOnlineOrientationRecap();
        break;
      case RecapsType.appointment:
        this.handleAppointmentRecap();
        break;
      case RecapsType.structure:
        this.handleStructureRecap();
        break;
      default:
        throw new Error(`Not implemented recap type ${this.recapType}`);
    }
  }

  public shouldDisplayAppointmentRecap(): boolean {
    return this.recapType === RecapsType.appointment;
  }

  public handleAppointmentRecap(): void {
    this.comment = this.form.get('details').value;
    this.orientator = this.form.get('structureOrientator').value;
  }

  public handleStructureRecap(): void {
    this.comment = this.form.get('comments').value;
    this.structuresToPrint = this.form.get('structureChoice').value;
    this.orientator = this.form.get('structureOrientator').value;
  }

  public isOrientator(): boolean {
    return Boolean(this.orientator?.structureName);
  }

  public handleOnlineOrientationRecap(): void {
    this.comment = this.form.get('comments').value;
    this.orientator = this.form.get('structureOrientator').value;
    const monthNumber = parseInt(this.form.get('dateSlot').value.day.slice(-2));
    const monthName = this.datePipe.transform(`2000-${monthNumber}-01`, 'MMMM', 'fr');

    this.recap = {
      day: this.form.get('dateSlot').value.day.slice(0, -3),
      hours: this.form.get('dateSlot').value.hours.replace('-', ' et '),
      month: monthName,
      language: this.form.get('preferredLanguage').value,
    };
  }

  public getRecapTitle(): string {
    switch (this.recapType) {
      case RecapsType.structure:
        return 'Orientation vers un lieu de médiation numérique';
      case RecapsType.appointment:
        return 'Demande de rendez-vous d’aide numérique';
      case RecapsType.onlineMediation:
        return 'Rappel téléphonique';
      default:
        throw new Error(`Not implemented recap type ${this.recapType}`);
    }
  }

  public getRecapInfo(): string | null {
    switch (this.recapType) {
      case RecapsType.structure:
        return null;
      case RecapsType.appointment:
        return 'Votre demande de rendez-vous d’aide numérique a bien été envoyée.<br>Un·e professionnel·le vous rappellera pour fixer une date.';
      case RecapsType.onlineMediation:
        return 'Vous avez effectué une demande de rappel téléphonique.<br>Vous serez contacté(e) par un·e professionnel·le pour le créneau sélectionné.';
      default:
        throw new Error(`Not implemented recap type ${this.recapType}`);
    }
  }

  public getFormattedNeeds(): string {
    return this.needs.map((n) => n.displayText).join(', ');
  }
}
